import { SimpleSchema } from 'meteor/aldeed:simple-schema';
import {
    studyDesignOptions,
    exposureAssessmentTypeOptions,
    ratings,
    biasDirection,
    biasDirectionPopoverText,
    ratingRationalePopoverText,
} from './constants';


export default {
    // #1: General
    referenceID: {
        label: 'Reference',
        type: SimpleSchema.RegEx.Id,
    },
    additionalReferences: {
        label: 'References',
        type: [SimpleSchema.RegEx.Id],
        minCount: 0,
        popoverText: 'References of earlier updates or related publications, such as publications on exposure assessment or other relevant findings from the study population',
    },
    studyDesign: {
        label: 'Study design',
        allowedValues: studyDesignOptions,
        type: String,
        popoverText: 'Study-design used for evaluation (Cohort, Case-Control, Nested-Case Control, etc.)',
    },
    location: {
        label: 'Location',
        type: String,
        optional: true,
        popoverText: 'Country; other information (e.g., region, state, province, city) if important',
    },
    enrollmentDates: {
        label: 'Enrollment dates',
        type: String,
        optional: true,
        popoverText: 'Enrollment date for cohort or case-controls',
    },
    // #2: Population
    populationEligibility: {
        label: 'Eligibility criteria',
        type: String,
        textAreaRows: 3,
        popoverText: 'Describe the inclusion/exclusion criteria  (e.g., age, sex, length of employment) for selecting the cohort. Provide information on the subcohort (exposed) as well as the total cohort',
        optional: true,
    },
    cohortPopulationSize: {
        label: 'Population size',
        type: String,
        popoverText: 'Number enrolled for cohort studies which were included in the analysis after exclusions',
        optional: true,
    },
    referentGroup: {
        label: 'Referent group type',
        type: String,
        popoverText: 'i.e., external vs. internal analysis (or NR if not reported)',
        optional: true,
    },
    lossToFollowUp: {
        label: 'Loss to follow up',
        type: String,
        popoverText: 'The proportion of enrolled subjects whose status was unknown at the end of follow-up',
        optional: true,
    },
    smrAllCauses: {
        label: 'SMR/SIR all causes/cases',
        type: String,
        popoverText: '<ADD>',
        optional: true,
    },
    otherPopulationDescriptors: {
        label: 'Other population descriptors',
        type: String,
        optional: true,
        textAreaRows: 5,
        popoverText: 'Provide a short definition to describe the cohort or case-control study - Nurses heath study, type of workers/geographical location',
    },
    selectionBiasRating: {
        label: 'Selection bias rating',
        type: String,
        allowedValues: ratings,
        popoverText: ratingRationalePopoverText,
        biasField: true,
        biasSummary: 'Population',
    },
    selectionBiasDirection:{
        label: 'Bias direction',
        type: String,
        allowedValues: biasDirection,
        popoverText: biasDirectionPopoverText,
        biasField: true,
    },
    selectionBiasRationale: {
        label: 'Selection bias rationale',
        type: String,
        optional: true,
        textAreaRows: 2,
        popoverText: ratingRationalePopoverText,
        biasField: true,
    },
    selectionDescriptionCases: {
        label: 'Selection description',
        type: String,
        popoverText: 'Describe the inclusion/exclusion criteria for selecting the cases (such as age, dates sex) and the source population (such as geographical location, cancer registry)',
        optional: true,
    },
    selectionDescriptionControls: {
        label: 'Selection description',
        type: String,
        popoverText: 'Describe the inclusion/exclusion criteria for selecting selection referent (such as age, dates sex) and the source population (such as geographical location, cancer registry)',
        optional: true,
    },
    populationSizeCases: {
        label: 'Population size',
        type: String,
        popoverText: 'Number of cases/controls in case-control study which were included in the analysis after exclusions',
        optional: true,
    },
    populationSizeControls: {
        label: 'Population size',
        type: String,
        popoverText: 'Number of cases/controls in case-control study which were included in the analysis after exclusions',
        optional: true,
    },
    responseRateCases: {
        label: 'Response rate',
        type: String,
        popoverText: 'Percent of eligible participants included, NR if not-reported',
        optional: true,
    },
    responseRateControls: {
        label: 'Response rate',
        type: String,
        popoverText: 'Percent of eligible participants included, NR if not-reported',
        optional: true,
    },
    // #3: Exposure/outcome
    exposureAssessmentType: {
        label: 'Exposure assessment type',
        type: String,
        allowedValues: exposureAssessmentTypeOptions,
        popoverText: 'Which method was used to estimate agent-exposure?',
        optional: true,
    },
    exposureAssessmentNotes: {
        label: 'Exposure assessment notes',
        type: String,
        textAreaRows: 8,
        popoverText: 'Approach used to estimate exposure, if any (quantitative measurement, JEM questionnaire, expert judgment, biomonitoring, other)',
        optional: true,
    },
    exposureAssessmentRating: {
        label: 'Exposure assessment rating',
        type: String,
        allowedValues: ratings,
        popoverText: ratingRationalePopoverText,
        biasField: true,
        biasSummary: 'Exposure',
    },
    exposureAssessmentDirection:{
        label: 'Bias direction',
        type: String,
        allowedValues: biasDirection,
        popoverText: biasDirectionPopoverText,
        biasField: true,
    },
    exposureAssessmentRationale: {
        label: 'Exposure assessment rationale',
        type: String,
        textAreaRows: 5,
        popoverText: ratingRationalePopoverText,
        optional: true,
        biasField: true,
    },
    exposureMissingData: {
        label: 'Exposure missing data',
        type: String,
        optional: true,
        textAreaRows: 5,
        popoverText: 'Note any missing exposure data for exposure to the candidate substance exposure (includes number of workers sampled if personal sampling used).',
    },
    outcomeDataSource: {
        label: 'Outcome source & methods',
        type: String,
        optional: true,
        textAreaRows: 6,
        popoverText: 'Method of follow-up or source of outcome data (e.g. linkage to national cancer registry, searches of death certificates, insurance records, etc.). Also note methods used to diagnosis disease and classification systems for disease endpoints (such as ICD code version).',
    },
    outcomeMissingData: {
        label: 'Outcome missing data',
        type: String,
        optional: true,
        textAreaRows: 6,
        popoverText: 'Note any missing data on disease.',
    },
    outcomeAssessmentRating: {
        label: 'Outcome assessment rating',
        type: String,
        allowedValues: ratings,
        popoverText: ratingRationalePopoverText,
        biasField: true,
        biasSummary: 'Exposure',
    },
    outcomeAssessmentDirection:{
        label: 'Bias direction',
        type: String,
        allowedValues: biasDirection,
        popoverText: biasDirectionPopoverText,
        biasField: true,
    },
    outcomeAssessmentRationale: {
        label: 'Outcome assessment rationale',
        type: String,
        optional: true,
        textAreaRows: 3,
        popoverText: ratingRationalePopoverText,
        biasField: true,
    },
    confidenceInEvidence: {
        label: 'Confidence in evidence',
        type: String,
        optional: true,
        textAreaRows: 3,
        popoverText: 'For each outcome (type of cancer), please rate the confidence of the evidence (evidence, some evidence, null, inconclusive, other) based on guidance handbook and/or protocol; please include the cancer type in your description',
        biasField: false,
    },
    // #4: Analysis/reporting
    analyticalMethods: {
        label: 'Analytical methods',
        type: String,
        optional: true,
        textAreaRows: 7,
        popoverText: 'Summarizes the statistical methods used and analyses conducted including information such as the models used, internal/external analyses, exposure-response relationships and statistics to calculate trends',
    },
    analysisRating: {
        label: 'Analysis rating',
        type: String,
        allowedValues: ratings,
        popoverText: ratingRationalePopoverText,
        biasField: true,
        biasSummary: 'Analysis',
    },
    analysisDirection:{
        label: 'Bias direction',
        type: String,
        allowedValues: biasDirection,
        popoverText: biasDirectionPopoverText,
        biasField: true,
    },
    analysisRationale: {
        label: 'Analysis rationale',
        type: String,
        optional: true,
        textAreaRows: 4,
        popoverText: ratingRationalePopoverText,
        biasField: true,
    },
    selectiveReportingRating: {
        label: 'Selective reporting rating',
        type: String,
        allowedValues: ratings,
        popoverText: ratingRationalePopoverText,
        biasField: true,
        biasSummary: 'Analysis',
    },
    selectiveReportingDirection:{
        label: 'Bias direction',
        type: String,
        allowedValues: biasDirection,
        popoverText: biasDirectionPopoverText,
        biasField: true,
    },
    selectiveReportingRationale: {
        label: 'Selective reporting rationale',
        type: String,
        optional: true,
        textAreaRows: 4,
        popoverText: ratingRationalePopoverText,
        biasField: true,
    },
    // #5: Sensitivity
    exposureLevel: {
        label: 'Exposure level(s)',
        type: String,
        optional: true,
        popoverText: 'Includes any measurements of levels (area versus population) or biomonitoring data (including time period) of exposure   Also include information on exposure duration, range, etc.',
    },
    exposedCasesOrPower: {
        label: 'Exposed cases or power',
        type: String,
        optional: true,
        textAreaRows: 4,
        popoverText: 'Include information related to statistical power such as exposed cases, prevalence of exposure, etc.',
    },
    followUp: {
        label: 'Follow-up',
        type: String,
        popoverText: 'Include calendar time period, average or range of follow up (years or person years) if available.',
        optional: true,
    },
    sensitivityOther: {
        label: 'Other sensitivity notes',
        type: String,
        optional: true,
        textAreaRows: 4,
        popoverText: 'Include any other types of information (such as analyses) that may effect the sensitivity of the study to detect a true effect.  See RoC handbook/protocol for candidate substances.',
    },
    sensitivityRating: {
        label: 'Sensitivity rating',
        type: String,
        allowedValues: ratings,
        popoverText: ratingRationalePopoverText,
        biasField: true,
        biasSummary: 'Sensitivity',
    },
    sensitivityDirection:{
        label: 'Bias direction',
        type: String,
        allowedValues: biasDirection,
        popoverText: biasDirectionPopoverText,
        biasField: true,
    },
    sensitivityRatingRationale: {
        label: 'Sensitivity rating rationale',
        type: String,
        optional: true,
        textAreaRows: 4,
        popoverText: ratingRationalePopoverText,
        biasField: true,
    },
    // #6: Study judgment
    strengths: {
        label: 'Principal strengths',
        type: String,
        optional: true,
        popoverText: ratingRationalePopoverText + '; note major strengths.',
        textAreaRows: 4,
        biasField: true,
    },
    limitations: {
        label: 'Principal limitations',
        type: String,
        optional: true,
        popoverText: ratingRationalePopoverText + '; note major limitations.',
        textAreaRows: 4,
        biasField: true,
    },
    overallUtility: {
        label: 'Overall utility',
        type: String,
        allowedValues: ratings,
        popoverText: ratingRationalePopoverText,
        biasField: true,
        biasSummary: 'Study judgement',
    },
};
